class ComponentRepository:
    """
    Repository class for component-specific database operations
    """
    
    def __init__(self, db_connection):
        """
        Initialize repository with database connection
        """
        self.db = db_connection

    def save_component(self, technical_name, functional_name, id_type, folder, class_name, status):
        """
        Inserts a new component into schmesys.component table
        
        Args:
            technical_name (str): Technical name of the component
            id_type (int): Component type ID
            status (int): Component status
        """
        cursor = None
        try:
            cursor = self.db.connection.cursor()
            
            insert_query = """
                INSERT INTO schmesys.component(technical_name,functional_name, id_subtype, folder, class_name, status) 
                VALUES (%s, %s, %s, %s, %s, %s)
                RETURNING id;
            """

            cursor.execute(insert_query, (technical_name, functional_name, id_type, folder, class_name, status))
            component_id = cursor.fetchone()[0]
            self.db.commit_transaction()
            
            return component_id
            
        except Exception as e:
            self.db.rollback_transaction()
            raise Exception(f"Error saving component '{technical_name}': {str(e)}")
        finally:
            if cursor:
                cursor.close()

    def query_components_by_type(self, id_type: int) -> list:
        """
        Consulta componentes de un tipo específico en la base de datos
        
        Args:
            id_type (int): ID del tipo de componente a consultar
        """
        cursor = None
        try:
            cursor = self.db.connection.cursor()
            
            query = """
            SELECT * 
            FROM schmesys.component 
            WHERE id_subtype = %s
            ORDER BY technical_name
            """
            
            cursor.execute(query, (id_type,))
            results = cursor.fetchall()
            
            components = []
            for row in results:
                component = {
                    'id': row[0],
                    'technical_name': row[1],
                    'id_type': row[2],
                    'status': row[3]
                }
                components.append(component)
            
            return components
            
        except Exception as e:
            raise Exception(f"Error querying components by type {id_type}: {str(e)}")
        finally:
            if cursor:
                cursor.close()




import boto3
import json

class Util:
    """Utility class for common operations"""
    
    @staticmethod
    def get_secret_aws(secret_name: str, logger, region_name: str):
        """
        Retrieves AWS Secrets Manager secret and returns database credentials
        """
        try:
            session = boto3.session.Session()
            client = session.client(
                service_name='secretsmanager',
                region_name=region_name
            )

            get_secret_value_response = client.get_secret_value(SecretId=secret_name)
            secret_string = get_secret_value_response['SecretString']
            secret_json = json.loads(secret_string)

            required_keys = ["password", "host", "port", "username", "dbname"]
            missing_keys = [key for key in required_keys if key not in secret_json]

            if missing_keys:
                msg = f"Missing required keys in AWS secret: {missing_keys}"
                logger.error(msg)
                raise KeyError(msg)

            return secret_json

        except Exception as e:
            msg = f"Error getting the secret '{secret_name}': {str(e)}"
            logger.error(msg)
            raise Exception(msg)





import psycopg2

class DatabaseConnection:
    """
    Class to handle PostgreSQL database connections
    using AWS Secrets Manager credentials
    """
    
    def __init__(self):
        self.connection = None

    def connect(self, credentials):
        """Establishes database connection using credentials"""
        try:
            self.connection = psycopg2.connect(
                host=credentials['host'],
                port=credentials['port'],
                database=credentials['dbname'],
                user=credentials['username'],
                password=credentials['password'],
                connect_timeout=30
            )
        except Exception as e:
            raise Exception(f"Database connection error: {str(e)}")

    def close(self):
        """Closes the database connection"""
        if self.connection and not self.connection.closed:
            self.connection.close()

    def commit_transaction(self):
        """Commits pending transactions"""
        if self.connection and not self.connection.closed:
            self.connection.commit()

    def rollback_transaction(self):
        """Rolls back pending transactions in case of error"""
        if self.connection and not self.connection.closed:
            self.connection.rollback()
